@@ -1,6 +1,7 @@
-(def ks-version "1.0.0")
+(def ks-version "1.3.0")
+(def logback-version "1.1.3")
 
-(defproject puppetlabs/trapperkeeper "1.2.0"
+(defproject puppetlabs/trapperkeeper "1.3.1"
   :description "A framework for configuring, composing, and running Clojure services."
   ;; Abort when version ranges or version conflicts are detected in
   ;; dependencies. Also supports :warn to simply emit warnings.
@@ -14,11 +15,20 @@
                  [prismatic/schema "1.0.4"]
                  [org.clojure/tools.nrepl "0.2.3"]
                  [org.clojure/tools.macro "0.1.2"]
-                 [ch.qos.logback/logback-classic "1.1.1"]
+                 [ch.qos.logback/logback-classic ~logback-version]
+                 ;; even though we don't strictly have a dependency on the following two
+                 ;; logback artifacts, specifying the dependency version here ensures
+                 ;; that downstream projects don't pick up different versions that would
+                 ;; conflict with our version of logback-classic
+                 [ch.qos.logback/logback-core ~logback-version]
+                 [ch.qos.logback/logback-access ~logback-version]
+                 [org.slf4j/log4j-over-slf4j "1.7.6"]
                  [org.codehaus.janino/janino "2.7.8"]
-                 [puppetlabs/typesafe-config "0.1.1"]
-                 [me.raynes/fs "1.4.5"]
-                 [clj-yaml "0.4.0"]]
+                 [puppetlabs/typesafe-config "0.1.5"]
+                 [me.raynes/fs "1.4.6"]
+                 [clj-yaml "0.4.0"]
+                 [beckon "0.1.1"]
+                 [org.clojure/core.async "0.2.374"]]
 
   :lein-release {:scm         :git
                  :deploy-via  :lein-deploy}
