@@ -18,53 +18,82 @@
    * create-var
    * var?
 
+   Setting up the global env is also required, see clojure.tools.analyzer.env
+
    See clojure.tools.analyzer.core-test for an example on how to setup the analyzer."
   (:refer-clojure :exclude [macroexpand-1 macroexpand var? record?])
-  (:require [clojure.tools.analyzer.utils :refer :all]))
+  (:require [clojure.tools.analyzer.utils :refer :all]
+            [clojure.tools.analyzer.env :as env])
+  (:import (clojure.lang Symbol IPersistentVector IPersistentMap IPersistentSet ISeq IType IRecord)))
 
-(defmulti -analyze (fn [op form env & _] op))
-(defmulti -parse
-  "Takes a form and an env map and dispatches on the head of the form, that is
-   a special form."
-  (fn [[op & rest] env] op)
-  :default :invoke)
+(derive :ctx/return :ctx/expr)
 
-(defn analyze-form
+(defmulti -analyze-form (fn [form _] (class form)))
+
+(declare analyze-symbol
+         analyze-vector
+         analyze-map
+         analyze-set
+         analyze-seq
+         analyze-const)
+
+(def ^:dynamic analyze-form
   "Like analyze, but does not mark the form with :top-level true"
+  -analyze-form)
+
+(defmethod -analyze-form Symbol
   [form env]
-  (let [form (if (seq? form)
-               (or (seq form) ()) ;; force evaluation for analysis
-               form)]
-    (cond
+  (analyze-symbol form env))
 
-     (symbol? form)          (-analyze :symbol form env)
+(defmethod -analyze-form IPersistentVector
+  [form env]
+  (analyze-vector form env))
 
-     (type? form)            (-analyze :const  form env :type)
-     (record? form)          (-analyze :const  form env :record) ;; since recors are maps too, record? *needs*
-     ;; to be before map?
-     (and (seq? form)
-          (not (empty? form))) (-analyze :seq    form env) ;; handles function/macro/special-form invocations
+(defmethod -analyze-form IPersistentMap
+  [form env]
+  (analyze-map form env))
 
-     (vector? form)          (-analyze :vector form env)
-     (map? form)             (-analyze :map    form env)
-     (set? form)             (-analyze :set    form env)
+(defmethod -analyze-form IPersistentSet
+  [form env]
+  (analyze-set form env))
 
-     :else                   (-analyze :const  form env))))
+(defmethod -analyze-form ISeq
+  [form env]
+  (if-let [form (seq form)]
+    (analyze-seq form env)
+    (analyze-const form env)))
+
+(defmethod -analyze-form IType
+  [form env]
+  (analyze-const form env :type))
+
+(prefer-method -analyze-form IType IPersistentMap)
+(prefer-method -analyze-form IType IPersistentVector)
+(prefer-method -analyze-form IType IPersistentSet)
+(prefer-method -analyze-form IType ISeq)
+
+(defmethod -analyze-form IRecord
+  [form env]
+  (analyze-const form env :record))
+
+(prefer-method -analyze-form IRecord IPersistentMap)
+(prefer-method -analyze-form IRecord IPersistentVector)
+(prefer-method -analyze-form IRecord IPersistentSet)
+(prefer-method -analyze-form IRecord ISeq)
+
+(defmethod -analyze-form :default
+  [form env]
+  (analyze-const form env))
 
 (defn analyze
   "Given a form to analyze and an environment, a map containing:
    * :locals     a map from binding symbol to AST of the binding value
-   * :context    a keyword describing the form's context, one of:
-    ** :return     the form is in return position
-    ** :statement  the return value of the form is not needed
-    ** :expr       everything else
+   * :context    a keyword describing the form's context from the :ctx/* hierarchy.
+    ** :ctx/expr      the form is an expression: its value is used
+    ** :ctx/return    the form is an expression in return position, derives :ctx/expr
+    ** :ctx/statement the value of the form is not used
    * :ns         a symbol representing the current namespace of the form to be
-                 analyzed, must be present in the :namespaces map
-   * :namespaces an atom containing a map from namespace symbol to namespace map,
-                 the namespace map contains the following keys:
-    ** :mappings   a map of mappings of the namespace, symbol to var/class
-    ** :aliases    a map of the aliases of the namespace, symbol to symbol
-    ** :ns         a symbol representing the namespace
+                 analyzed
 
    returns an AST for that form.
 
@@ -88,13 +117,9 @@
 (defn empty-env
   "Returns an empty env"
   []
-  {:context    :expr
+  {:context    :ctx/expr
    :locals     {}
-   :ns         'user
-   :namespaces (atom
-                {'user {:mappings {}
-                        :aliases  {}
-                        :ns       'user}})})
+   :ns         'user})
 
 (defn analyze-in-env
   "Takes an env map and returns a function that analyzes a form in that env"
@@ -132,15 +157,14 @@
       {:op       :with-meta
        :env      env
        :form     form
-       :meta     (analyze-form meta (ctx env :expr))
-       :expr     (assoc-in expr [:env :context] :expr)
+       :meta     (analyze-form meta (ctx env :ctx/expr))
+       :expr     (assoc-in expr [:env :context] :ctx/expr)
        :children [:meta :expr]}
       expr)))
 
-(defmethod -analyze :const
-  [_ form env & [type]]
-  (let [type (or type (classify form))
-        m (meta form)]
+(defn analyze-const
+  [form env & [type]]
+  (let [type (or type (classify form))]
     (merge
      {:op       :const
       :env      env
@@ -148,15 +172,14 @@
       :literal? true
       :val      form
       :form     form}
-     (when (and (obj? form)
-                (seq m))
-       {:meta     (-analyze :const m (ctx env :expr) :map) ;; metadata on a constant literal will not be evaluated at
-                                                           ;; runtime, this is also true for metadata on quoted collection literals
-        :children [:meta]}))))
+     (when-let [m (and (obj? form)
+                       (not-empty (meta form)))]
+       {:meta     (analyze-const m (ctx env :ctx/expr) :map) ;; metadata on a constant literal will not be evaluated at
+        :children [:meta]}))))                               ;; runtime, this is also true for metadata on quoted collection literals
 
-(defmethod -analyze :vector
-  [_ form env]
-  (let [items-env (ctx env :expr)
+(defn analyze-vector
+  [form env]
+  (let [items-env (ctx env :ctx/expr)
         items (mapv (analyze-in-env items-env) form)]
     (wrapping-meta
      {:op       :vector
@@ -165,9 +188,9 @@
       :form     form
       :children [:items]})))
 
-(defmethod -analyze :map
-  [_ form env]
-  (let [kv-env (ctx env :expr)
+(defn analyze-map
+  [form env]
+  (let [kv-env (ctx env :ctx/expr)
         [keys vals] (reduce-kv (fn [[keys vals] k v]
                                  [(conj keys k) (conj vals v)])
                                [[] []] form)
@@ -181,9 +204,9 @@
       :form     form
       :children [:keys :vals]})))
 
-(defmethod -analyze :set
-  [_ form env]
-  (let [items-env (ctx env :expr)
+(defn analyze-set
+  [form env]
+  (let [items-env (ctx env :ctx/expr)
         items (mapv (analyze-in-env items-env) form)]
     (wrapping-meta
      {:op       :set
@@ -194,8 +217,8 @@
 
 (def specials
   "Set of special forms common to every clojure variant"
-  '#{do if new quote set! try
-     catch throw finally & def .
+  '#{do if new quote set! try var
+     catch throw finally def .
      let* letfn* loop* recur fn*})
 
 (defn macroexpand
@@ -208,55 +231,51 @@
         mform
         (recur mform)))))
 
-(defmethod -analyze :symbol
-  [_ sym env]
+(defn analyze-symbol
+  [sym env]
   (let [mform (macroexpand-1 sym env)] ;; t.a.j/macroexpand-1 macroexpands Class/Field into (. Class Field)
     (if (= mform sym)
       (merge (if-let [{:keys [mutable children] :as local-binding} (-> env :locals sym)] ;; locals shadow globals
-               (merge (dissoc local-binding :init) ;; avoids useless passes later
+               (merge (dissoc local-binding :init)                                      ;; avoids useless passes later
                       {:op          :local
                        :assignable? (boolean mutable)
                        :children    (vec (remove #{:init} children))})
-               (if-let [var (let [v (resolve-var sym env)]
+               (if-let [var (let [v (resolve-sym sym env)]
                               (and (var? v) v))]
-                 {:op          :var
-                  :assignable? (dynamic? var) ;; we cannot statically determine if a Var is in a thread-local context
-                                              ;; so checking whether it's dynamic or not is the most we can do
-                  :var         var}
+                 (let [m (meta var)]
+                   {:op          :var
+                    :assignable? (dynamic? var m) ;; we cannot statically determine if a Var is in a thread-local context
+                    :var         var              ;; so checking whether it's dynamic or not is the most we can do
+                    :meta        m})
                  (if-let [maybe-class (namespace sym)] ;; e.g. js/foo.bar or Long/MAX_VALUE
                    (let [maybe-class (symbol maybe-class)]
-                     (if-not (resolve-ns maybe-class env) ;; namespaces shadow classes
-                       {:op    :maybe-host-form
-                        :class maybe-class
-                        :field (symbol (name sym))}
-                       (throw (ex-info (str "Could not resolve var: " sym)
-                                       (merge {:var mform}
-                                              (-source-info sym env))))))
+                     {:op    :maybe-host-form
+                      :class maybe-class
+                      :field (symbol (name sym))})
                    {:op    :maybe-class ;; e.g. java.lang.Integer or Long
                     :class mform})))
              {:env  env
               :form mform})
-      ;; should this preserve the original form?
-      (analyze-form (if (obj? mform)
-                      (with-meta mform (meta sym))
-                      mform)
-               env))))
+      (-> (analyze-form mform env)
+        (update-in [:raw-forms] (fnil conj ()) sym)))))
 
-(defmethod -analyze :seq
-  [_ form env]
+(defn analyze-seq
+  [form env]
   (let [op (first form)]
     (when (nil? op)
       (throw (ex-info "Can't call nil"
                       (merge {:form form}
                              (-source-info form env)))))
     (let [mform (macroexpand-1 form env)]
-      (if (= form mform)  ;; function/special-form invocation
+      (if (= form mform) ;; function/special-form invocation
         (parse mform env)
-        (analyze-form mform env)))))
+        (-> (analyze-form mform env)
+          (update-in [:raw-forms] (fnil conj ())
+                     (vary-meta form assoc ::resolved-op (resolve-sym op env))))))))
 
-(defmethod -parse 'do
+(defn parse-do
   [[_ & exprs :as form] env]
-  (let [statements-env (ctx env :statement)
+  (let [statements-env (ctx env :ctx/statement)
         [statements ret] (loop [statements [] [e & exprs] exprs]
                            (if (seq exprs)
                              (recur (conj statements e) exprs)
@@ -270,13 +289,14 @@
      :ret        ret
      :children   [:statements :ret]}))
 
-(defmethod -parse 'if
+(defn parse-if
   [[_ test then else :as form] env]
-  (when-not (<= 3 (count form) 4)
-    (throw (ex-info (str "Wrong number of args to if, had: " (dec (count form)))
-                    (merge {:form form}
-                           (-source-info form env)))))
-  (let [test-expr (analyze-form test (ctx env :expr))
+  (let [formc (count form)]
+    (when-not (or (= formc 3) (= formc 4))
+      (throw (ex-info (str "Wrong number of args to if, had: " (dec (count form)))
+                      (merge {:form form}
+                             (-source-info form env))))))
+  (let [test-expr (analyze-form test (ctx env :ctx/expr))
         then-expr (analyze-form then env)
         else-expr (analyze-form else env)]
     {:op       :if
@@ -287,28 +307,28 @@
      :else     else-expr
      :children [:test :then :else]}))
 
-(defmethod -parse 'new
+(defn parse-new
   [[_ class & args :as form] env]
   (when-not (>= (count form) 2)
     (throw (ex-info (str "Wrong number of args to new, had: " (dec (count form)))
                     (merge {:form form}
                            (-source-info form env)))))
-  (let [args-env (ctx env :expr)
+  (let [args-env (ctx env :ctx/expr)
         args (mapv (analyze-in-env args-env) args)]
     {:op          :new
      :env         env
      :form        form
-     :class       class
+     :class       (analyze-form class (assoc env :locals {})) ;; avoid shadowing
      :args        args
-     :children    [:args]}))
+     :children    [:class :args]}))
 
-(defmethod -parse 'quote
+(defn parse-quote
   [[_ expr :as form] env]
   (when-not (= 2 (count form))
     (throw (ex-info (str "Wrong number of args to quote, had: " (dec (count form)))
                     (merge {:form form}
                            (-source-info form env)))))
-  (let [const (-analyze :const expr env)]
+  (let [const (analyze-const expr env)]
     {:op       :quote
      :expr     const
      :form     form
@@ -316,14 +336,14 @@
      :literal? true
      :children [:expr]}))
 
-(defmethod -parse 'set!
+(defn parse-set!
   [[_ target val :as form] env]
   (when-not (= 3 (count form))
     (throw (ex-info (str "Wrong number of args to set!, had: " (dec (count form)))
                     (merge {:form form}
                            (-source-info form env)))))
-  (let [target (analyze-form target (ctx env :expr))
-        val (analyze-form val (ctx env :expr))]
+  (let [target (analyze-form target (ctx env :ctx/expr))
+        val (analyze-form val (ctx env :ctx/expr))]
     {:op       :set!
      :env      env
      :form     form
@@ -338,15 +358,25 @@
 (defn valid-binding-symbol? [s]
   (and (symbol? s)
        (not (namespace s))
-       (not (.contains (name s) "."))))
+       (not (re-find #"\." (name s)))))
 
-(defmethod -parse 'try
+(defn ^:private split-with' [pred coll]
+  (loop [take [] drop coll]
+    (if (seq drop)
+      (let [[el & r] drop]
+        (if (pred el)
+          (recur (conj take el) r)
+          [(seq take) drop]))
+      [(seq take) ()])))
+
+(declare parse-catch)
+(defn parse-try
   [[_ & body :as form] env]
   (let [catch? (every-pred seq? #(= (first %) 'catch))
         finally? (every-pred seq? #(= (first %) 'finally))
-        [body tail'] (split-with (complement (some-fn catch? finally?)) body)
-        [cblocks tail] (split-with catch? tail')
-        [[fblock & fbs :as fblocks] tail] (split-with finally? tail)]
+        [body tail'] (split-with' (complement (some-fn catch? finally?)) body)
+        [cblocks tail] (split-with' catch? tail')
+        [[fblock & fbs :as fblocks] tail] (split-with' finally? tail)]
     (when-not (empty? tail)
       (throw (ex-info "Only catch or finally clause can follow catch in try expression"
                       (merge {:expr tail
@@ -357,45 +387,44 @@
                       (merge {:expr fblocks
                               :form form}
                              (-source-info form env)))))
-    (if (and (empty? cblocks)
-             (empty? fblocks))
-      (assoc (analyze-body body env) :form form) ;; discard the useless try
-      (let [body (analyze-body body (assoc env :no-recur true :in-try true)) ;; cannot recur across try
-            cenv (ctx env :expr)
-            cblocks (mapv #(parse % cenv) cblocks)
-            fblock (when-not (empty? fblock)
-                     (analyze-body (rest fblock) (ctx env :statement)))]
-        (merge {:op      :try
-                :env     env
-                :form    form
-                :body    body
-                :catches cblocks}
-               (when fblock
-                 {:finally fblock})
-               {:children `[:body :catches ~@(when fblock [:finally])]})))))
+    (let [env' (assoc env :in-try true)
+          body (analyze-body body env')
+          cenv (ctx env' :ctx/expr)
+          cblocks (mapv #(parse-catch % cenv) cblocks)
+          fblock (when-not (empty? fblock)
+                   (analyze-body (rest fblock) (ctx env :ctx/statement)))]
+      (merge {:op      :try
+              :env     env
+              :form    form
+              :body    body
+              :catches cblocks}
+             (when fblock
+               {:finally fblock})
+             {:children (into [:body :catches]
+                              (when fblock [:finally]))}))))
 
-(defmethod -parse 'catch
+(defn parse-catch
   [[_ etype ename & body :as form] env]
   (when-not (valid-binding-symbol? ename)
     (throw (ex-info (str "Bad binding form: " ename)
                     (merge {:sym ename
                             :form form}
                            (-source-info form env)))))
-  (let [local {:op    :binding
+  (let [env (dissoc env :in-try)
+        local {:op    :binding
                :env   env
                :form  ename
                :name  ename
-               :local :catch
-               :tag   etype}]
+               :local :catch}]
     {:op          :catch
-     :class       etype
+     :class       (analyze-form etype (assoc env :locals {}))
      :local       local
      :env         env
      :form        form
      :body        (analyze-body body (assoc-in env [:locals ename] (dissoc-env local)))
-     :children    [:local :body]}))
+     :children    [:class :local :body]}))
 
-(defmethod -parse 'throw
+(defn parse-throw
   [[_ throw :as form] env]
   (when-not (= 2 (count form))
     (throw (ex-info (str "Wrong number of args to throw, had: " (dec (count form)))
@@ -404,7 +433,7 @@
   {:op        :throw
    :env       env
    :form      form
-   :exception (analyze-form throw (ctx env :expr))
+   :exception (analyze-form throw (ctx env :ctx/expr))
    :children  [:exception]})
 
 (defn validate-bindings
@@ -423,12 +452,11 @@
                             :bindings bindings}
                            (-source-info form env))))))
 
-(defmethod -parse 'letfn*
+(defn parse-letfn*
   [[_ bindings & body :as form] env]
   (validate-bindings form env)
-  (let [bindings (apply hash-map bindings) ;; non-determinalistically pick only one local with the same name,
-                                           ;; if more are present.
-        fns (keys bindings)]
+  (let [bindings (apply array-map bindings) ;; pick only one local with the same name, if more are present.
+        fns      (keys bindings)]
     (when-let [[sym] (seq (remove valid-binding-symbol? fns))]
       (throw (ex-info (str "Bad binding form: " sym)
                       (merge {:form form
@@ -447,7 +475,7 @@
                              (assoc binds name
                                     (merge bind
                                            {:init     (analyze-form (bindings name)
-                                                               (ctx e :expr))
+                                                                    (ctx e :ctx/expr))
                                             :children [:init]})))
                            {} binds)
           e (update-in env [:locals] merge (update-vals binds dissoc-env))
@@ -464,7 +492,7 @@
   (validate-bindings form env)
   (let [loop? (= 'loop* op)]
     (loop [bindings bindings
-           env (ctx env :expr)
+           env (ctx env :ctx/expr)
            binds []]
       (if-let [[name init & bindings] (seq bindings)]
         (if (not (valid-binding-symbol? name))
@@ -483,7 +511,7 @@
             (recur bindings
                    (assoc-in env [:locals name] (dissoc-env bind-expr))
                    (conj binds bind-expr))))
-        (let [body-env (assoc env :context (if loop? :return context))
+        (let [body-env (assoc env :context (if loop? :ctx/return context))
               body (analyze-body body (merge body-env
                                              (when loop?
                                                {:loop-id     loop-id
@@ -492,34 +520,31 @@
            :bindings binds
            :children [:bindings :body]})))))
 
-(defmethod -parse 'let*
+(defn parse-let*
   [form env]
   (into {:op   :let
          :form form
          :env  env}
         (analyze-let form env)))
 
-(defmethod -parse 'loop*
+(defn parse-loop*
   [form env]
   (let [loop-id (gensym "loop_") ;; can be used to find matching recur
-        env (dissoc (assoc env :loop-id loop-id) :no-recur)]
+        env (assoc env :loop-id loop-id)]
     (into {:op      :loop
            :form    form
            :env     env
            :loop-id loop-id}
           (analyze-let form env))))
 
-(defmethod -parse 'recur
-  [[_ & exprs :as form] {:keys [context loop-locals loop-id no-recur]
+(defn parse-recur
+  [[_ & exprs :as form] {:keys [context loop-locals loop-id]
                          :as env}]
   (when-let [error-msg
              (cond
-              (not (= :return context))
+              (not (isa? context :ctx/return))
               "Can only recur from tail position"
 
-              no-recur
-              "Cannot recur across try"
-
               (not (= (count exprs) loop-locals))
               (str "Mismatched argument count to recur, expected: " loop-locals
                    " args, had: " (count exprs)))]
@@ -528,7 +553,7 @@
                             :form  form}
                            (-source-info form env)))))
 
-  (let [exprs (mapv (analyze-in-env (ctx env :expr)) exprs)]
+  (let [exprs (mapv (analyze-in-env (ctx env :ctx/expr)) exprs)]
     {:op          :recur
      :env         env
      :form        form
@@ -537,6 +562,12 @@
      :children    [:exprs]}))
 
 (defn analyze-fn-method [[params & body :as form] {:keys [locals local] :as env}]
+  (when-not (vector? params)
+    (throw (ex-info "Parameter declaration should be a vector"
+                    (merge {:params params
+                            :form   form}
+                           (-source-info form env)
+                           (-source-info params env)))))
   (when (not-every? valid-binding-symbol? params)
     (throw (ex-info (str "Params must be valid binding symbols, had: "
                          (mapv class params))
@@ -544,14 +575,8 @@
                             :form   form}
                            (-source-info form env)
                            (-source-info params env))))) ;; more specific
-  (when-not (vector? params)
-    (throw (ex-info "Parameter declaration should be a vector"
-                    (merge {:params params
-                            :form   form}
-                           (-source-info form env)
-                           (-source-info params env)))))
   (let [variadic? (boolean (some '#{&} params))
-        params-names (if variadic? (vec (remove '#{&} params)) params)
+        params-names (if variadic? (conj (pop (pop params)) (peek params)) params)
         env (dissoc env :local)
         arity (count params-names)
         params-expr (mapv (fn [name id]
@@ -570,7 +595,7 @@
         loop-id (gensym "loop_")
         body-env (into (update-in env [:locals]
                                   merge (zipmap params-names (map dissoc-env params-expr)))
-                       {:context     :return
+                       {:context     :ctx/return
                         :loop-id     loop-id
                         :loop-locals (count params-expr)})
         body (analyze-body body body-env)]
@@ -600,56 +625,57 @@
       :body        body
       :children    [:params :body]}
      (when local
-       {:local (dissoc local :env)}))))
+       {:local (dissoc-env local)}))))
 
-(defmethod -parse 'fn*
+(defn parse-fn*
   [[op & args :as form] env]
-  (let [[n meths] (if (symbol? (first args))
-                    [(first args) (next args)]
-                    [nil (seq args)])
-        name-expr {:op    :binding
-                   :env   env
-                   :form  n
-                   :local :fn
-                   :name  n}
-        e (if n (assoc (assoc-in env [:locals n] (dissoc-env name-expr)) :local name-expr) env)
-        once? (-> op meta :once boolean)
-        menv (assoc (dissoc e :no-recur :in-try) :once once?)
-        meths (if (vector? (first meths)) (list meths) meths) ;;turn (fn [] ...) into (fn ([]...))
-        methods-exprs (mapv #(analyze-fn-method % menv) meths)
-        variadic (seq (filter :variadic? methods-exprs))
-        variadic? (boolean variadic)
-        fixed-arities (seq (map :fixed-arity (remove :variadic? methods-exprs)))
-        max-fixed-arity (when fixed-arities (apply max fixed-arities))]
-    (when (>= (count variadic) 2)
-      (throw (ex-info "Can't have more than 1 variadic overload"
-                      (merge {:variadics (mapv :form variadic)
-                              :form      form}
-                             (-source-info form env)))))
-    (when (not= (seq (distinct fixed-arities)) fixed-arities)
-      (throw (ex-info "Can't have 2 or more overloads with the same arity"
-                      (merge {:form form}
-                             (-source-info form env)))))
-    (when (and variadic?
-               (not-every? #(<= (:fixed-arity %)
-                          (:fixed-arity (first variadic)))
-                      (remove :variadic? methods-exprs)))
-      (throw (ex-info "Can't have fixed arity overload with more params than variadic overload"
-                      (merge {:form form}
-                             (-source-info form env)))))
-    (merge {:op              :fn
-            :env             env
-            :form            form
-            :variadic?       variadic?
-            :max-fixed-arity max-fixed-arity
-            :methods         methods-exprs
-            :once            once?}
-           (when n
-             {:local name-expr})
-           {:children `[~@(when n [:local]) :methods]})))
+  (wrapping-meta
+   (let [[n meths] (if (symbol? (first args))
+                     [(first args) (next args)]
+                     [nil (seq args)])
+         name-expr {:op    :binding
+                    :env   env
+                    :form  n
+                    :local :fn
+                    :name  n}
+         e (if n (assoc (assoc-in env [:locals n] (dissoc-env name-expr)) :local name-expr) env)
+         once? (-> op meta :once boolean)
+         menv (assoc (dissoc e :in-try) :once once?)
+         meths (if (vector? (first meths)) (list meths) meths) ;;turn (fn [] ...) into (fn ([]...))
+         methods-exprs (mapv #(analyze-fn-method % menv) meths)
+         variadic (seq (filter :variadic? methods-exprs))
+         variadic? (boolean variadic)
+         fixed-arities (seq (map :fixed-arity (remove :variadic? methods-exprs)))
+         max-fixed-arity (when fixed-arities (apply max fixed-arities))]
+     (when (>= (count variadic) 2)
+       (throw (ex-info "Can't have more than 1 variadic overload"
+                       (merge {:variadics (mapv :form variadic)
+                               :form      form}
+                              (-source-info form env)))))
+     (when (not= (seq (distinct fixed-arities)) fixed-arities)
+       (throw (ex-info "Can't have 2 or more overloads with the same arity"
+                       (merge {:form form}
+                              (-source-info form env)))))
+     (when (and variadic?
+                (not-every? #(<= (:fixed-arity %)
+                           (:fixed-arity (first variadic)))
+                       (remove :variadic? methods-exprs)))
+       (throw (ex-info "Can't have fixed arity overload with more params than variadic overload"
+                       (merge {:form form}
+                              (-source-info form env)))))
+     (merge {:op              :fn
+             :env             env
+             :form            form
+             :variadic?       variadic?
+             :max-fixed-arity max-fixed-arity
+             :methods         methods-exprs
+             :once            once?}
+            (when n
+              {:local name-expr})
+            {:children (conj (if n [:local] []) :methods)}))))
 
-(defmethod -parse 'def
-  [[_ sym & expr :as form] {:keys [ns namespaces] :as env}]
+(defn parse-def
+  [[_ sym & expr :as form] {:keys [ns] :as env}]
   (when (not (symbol? sym))
     (throw (ex-info (str "First argument to def must be a symbol, had: " (class sym))
                     (merge {:form form}
@@ -682,18 +708,20 @@
                      (-source-info form env)))
 
         var (create-var sym env) ;; interned var will have quoted arglists, replaced on evaluation
-        _ (swap! namespaces assoc-in [ns :mappings sym] var)
+        _ (env/deref-env) ;; make sure *env* is bound
+        _ (swap! env/*env* assoc-in [:namespaces ns :mappings sym] var)
 
         meta (merge (meta sym)
                     (when arglists
                       {:arglists (list 'quote arglists)}))
 
-        meta-expr (when meta (analyze-form meta (ctx env :expr))) ;; meta on def sym will be evaluated
+        meta-expr (when meta (analyze-form meta (ctx env :ctx/expr))) ;; meta on def sym will be evaluated
 
         args (when-let [[_ init] (find args :init)]
-               (merge args {:init (analyze-form init (ctx env :expr))}))
-        children `[~@(when meta [:meta])
-                   ~@(when (:init args) [:init])]]
+               (assoc args :init (analyze-form init (ctx env :ctx/expr))))
+        init? (:init args)
+        children (into (into [] (when meta [:meta]))
+                       (when init? [:init]))]
 
     (merge {:op   :def
             :env  env
@@ -706,7 +734,7 @@
            (when-not (empty? children)
              {:children children}))))
 
-(defmethod -parse '.
+(defn parse-dot
   [[_ target & [m-or-f & args] :as form] env]
   (when-not (>= (count form) 3)
     (throw (ex-info (str "Wrong number of args to ., had: " (dec (count form)))
@@ -716,7 +744,7 @@
                                  (= \- (first (name m-or-f))))
                           [(-> m-or-f name (subs 1) symbol) true]
                           [(if args (cons m-or-f args) m-or-f) false])
-        target-expr (analyze-form target (ctx env :expr))
+        target-expr (analyze-form target (ctx env :ctx/expr))
         call? (and (not field?) (seq? m-or-f))]
 
     (when (and call? (not (symbol? (first m-or-f))))
@@ -731,30 +759,68 @@
             call?
             {:op       :host-call
              :method   (symbol (name (first m-or-f)))
-             :args     (mapv (analyze-in-env (ctx env :expr)) (next m-or-f))
+             :args     (mapv (analyze-in-env (ctx env :ctx/expr)) (next m-or-f))
              :children [:target :args]}
 
             field?
-            {:op       :host-field
-             :field    (symbol (name m-or-f))
-             :children [:target]}
+            {:op          :host-field
+             :assignable? true
+             :field       (symbol (name m-or-f))
+             :children    [:target]}
 
             :else
-            {:op       :host-interop ;; either field access or no-args method call
-             :m-or-f   (symbol (name m-or-f))
-             :children [:target]}))))
+            {:op          :host-interop ;; either field access or no-args method call
+             :assignable? true
+             :m-or-f      (symbol (name m-or-f))
+             :children    [:target]}))))
 
-(defmethod -parse :invoke
+(defn parse-invoke
   [[f & args :as form] env]
-  (let [e (ctx env :expr)
-        fn-expr (analyze-form f e)
-        args-expr (mapv (analyze-in-env e) args)
+  (let [fenv (ctx env :ctx/expr)
+        fn-expr (analyze-form f fenv)
+        args-expr (mapv (analyze-in-env fenv) args)
         m (meta form)]
     (merge {:op   :invoke
             :form form
             :env  env
             :fn   fn-expr
             :args args-expr}
-           (when m
+           (when (seq m)
              {:meta m}) ;; meta on invoke form will not be evaluated
            {:children [:fn :args]})))
+
+(defn parse-var
+  [[_ var :as form] env]
+  (when-not (= 2 (count form))
+    (throw (ex-info (str "Wrong number of args to var, had: " (dec (count form)))
+                    (merge {:form form}
+                           (-source-info form env)))))
+  (if-let [var (resolve-sym var env)]
+    {:op   :the-var
+     :env  env
+     :form form
+     :var  var}
+    (throw (ex-info (str "var not found: " var) {:var var}))))
+
+(defn -parse
+  "Takes a form and an env map and dispatches on the head of the form, that is
+   a special form."
+  [form env]
+  ((case (first form)
+     do      parse-do
+     if      parse-if
+     new     parse-new
+     quote   parse-quote
+     set!    parse-set!
+     try     parse-try
+     throw   parse-throw
+     def     parse-def
+     .       parse-dot
+     let*    parse-let*
+     letfn*  parse-letfn*
+     loop*   parse-loop*
+     recur   parse-recur
+     fn*     parse-fn*
+     var     parse-var
+     #_:else parse-invoke)
+   form env))
